import React, { Fragment } from 'react'
import { get } from 'lodash'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import { actions } from 'pubsweet-client'
import { withJournal } from 'xpub-journal'
import { ConnectPage } from 'xpub-connect'
import { DragDropContext } from 'react-dnd'
import HTML5Backend from 'react-dnd-html5-backend'
import { Button, Spinner, Steps } from '@pubsweet/ui'
import { selectCollection, selectFragment } from 'xpub-selectors'
import {
  compose,
  toClass,
  withProps,
  withHandlers,
  withStateHandlers,
} from 'recompose'
import { Row, MultiAction } from 'pubsweet-component-faraday-ui'
import { withModal } from 'pubsweet-component-modal/src/components'
import { getUserToken } from 'pubsweet-component-faraday-selectors/src'

import {
  reduxForm,
  getFormValues,
  hasSubmitFailed,
  getFormSyncErrors,
  change as changeForm,
} from 'redux-form'
import {
  addAuthor,
  getAuthorFetching,
} from 'pubsweet-components-faraday/src/redux/authors'
import {
  uploadFile,
  deleteFile,
  getSignedUrl,
  getFileFetching,
} from 'pubsweet-components-faraday/src/redux/files'

import { wizardSteps } from './'
import { submitManuscript } from '../redux/conversion'
import { getAutosaveFetching } from '../redux/autosave'
import { onChange, onSubmit, setInitialValues, validate } from './utils'

// #region wizard
const Wizard = ({
  step,
  history,
  prevStep,
  isEditMode,
  isLastStep,
  isFirstStep,
  handleSubmit,
  getButtonText,
  isFilesFetching,
  isAuthorsFetching,
  journal: { manuscriptTypes = [] },
  ...rest
}) => (
  <Root className="wizard-root">
    <Steps currentStep={step}>
      {wizardSteps.map(({ stepTitle }) => (
        <Steps.Step key={stepTitle} title={stepTitle} />
      ))}
    </Steps>

    <StepRoot className="wizard-step">
      {wizardSteps[step].component({ manuscriptTypes, ...rest })}
      <Row justify="center" mt={2}>
        {isAuthorsFetching || isFilesFetching ? (
          <Spinner />
        ) : (
          <Fragment>
            {!isFirstStep && (
              <Button
                data-test="submission-back"
                mr={1}
                onClick={prevStep}
              >{`< BACK`}</Button>
            )}
            <Button
              data-test="submission-next"
              ml={isFirstStep ? 0 : 1}
              onClick={handleSubmit}
              primary
            >
              {getButtonText()}
            </Button>
          </Fragment>
        )}
      </Row>
    </StepRoot>
  </Root>
)
// #endregion

// #region export
export default compose(
  ConnectPage(({ match }) => [
    actions.getCollection({ id: match.params.project }),
    actions.getFragment(
      { id: match.params.project },
      { id: match.params.version },
    ),
  ]),
  withJournal,
  connect(
    (state, { match }) => ({
      token: getUserToken(state),
      isFetching: getAutosaveFetching(state),
      formValues: getFormValues('submission')(state),
      submitFailed: hasSubmitFailed('submission')(state),
      formSyncErrors: getFormSyncErrors('submission')(state),
      version: selectFragment(state, get(match, 'params.version')),
      project: selectCollection(state, get(match, 'params.project')),
      isFilesFetching: getFileFetching(state),
      isAuthorsFetching: getAuthorFetching(state) || getAutosaveFetching(state),
    }),
    {
      addAuthor,
      changeForm,
      uploadFile,
      deleteFile,
      getSignedUrl,
      submitManuscript,
      updateFragment: actions.updateFragment,
    },
  ),
  withStateHandlers(
    { step: 0 },
    {
      nextStep: ({ step }) => () => ({
        step: Math.min(wizardSteps.length - 1, step + 1),
      }),
      prevStep: ({ step }) => () => ({ step: Math.max(0, step - 1) }),
    },
  ),
  withProps(setInitialValues),
  withProps(({ formValues, formSyncErrors, submitFailed, step, location }) => ({
    isFirstStep: step === 0,
    isLastStep: step === wizardSteps.length - 1,
    isEditMode: get(location, 'state.editMode', false),
    filesError: submitFailed && get(formSyncErrors, 'files', ''),
    authorsError: submitFailed && get(formSyncErrors, 'authors', ''),
    hasConflicts: get(formValues, 'conflicts.hasConflicts', 'no') === 'yes',
  })),
  withHandlers({
    getButtonText: ({ isLastStep, isEditMode }) => () => {
      if (isEditMode && isLastStep) {
        return 'SAVE CHANGES'
      }
      return isLastStep ? `SUBMIT MANUSCRIPT` : `NEXT STEP >`
    },
  }),
  withModal(({ isFetching }) => ({
    isFetching,
    modalComponent: MultiAction,
  })),
  reduxForm({
    validate,
    onChange,
    onSubmit,
    form: 'submission',
  }),
  DragDropContext(HTML5Backend),
  toClass,
)(Wizard)
// #endregion

// #region styled-components
const Root = styled.div`
  align-items: center;
  display: flex;
  flex-direction: column;
  padding-bottom: calc(${th('gridUnit')} * 3);
`

const StepRoot = styled.div`
  align-items: center;
  background-color: ${th('wizard.colorBackground')};
  border-radius: ${th('borderRadius')};
  box-shadow: ${th('boxShadow')};
  display: flex;
  flex-direction: column;
  margin-top: ${th('gridUnit')};
  padding: calc(${th('gridUnit')} * 5) calc(${th('gridUnit')} * 4);
  position: relative;
  width: 100%;
`
// #endregion
